/****************************************************************************
#	 	jpcat: grabpicture from the spca5xx module via v4l device           #
#.                                                                          #
# 		Copyright (C) 21st century Jan Panteltje                            #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 2 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program; if not, write to the Free Software               #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA #
#                                                                           #
****************************************************************************/

#include "jpcat.h"


int restart_flag;
int quiet_flag;


void catch_alarm(int sig)
{
if(debug_flag)
	{
	fprintf(stderr, "catch_alarm(): sig=%d\n", sig);
	}

restart_flag = 1; 
signal (sig, catch_alarm);
} /* end function cacth _alarm */
             


int print_usage()
{
fprintf(stderr, "\
Usage:\n\
jpcat [-a] [-b brightness] [-c clockpos] [-d video_device] [-f fps] [-g] [-h] [-j] [-m motion_detection_sensitivity] [-o] [-p] [-q] [-r] [-s] [-t] [-u] [-v] [-w contrast] [-x] [-y]\n\n\
-a                   auto contrast on\n\
-b int               brightness\n\
-c int               clock position, 0=top_left, 1=top_middle, 2=top_right,\n\
                      3=bottom_left, 4=bottom_middle, 5=bottom_right, default bottom_middle\n\
-d string            video device, default /dev/video0\n\
-e 0/1               0=auto exposure off, 1=auto_exposure on (default), use modprobe gspca autoexpo=0 to fix going to dark, for module status do: ls -rtl /sys/module/gspca/parameters.\n\
-f float             speed in frames per second, default %.2f\n\
                      please set camera to auto in video setup\n\
-g                   grab a single frame and write it to stdout in jpeg format, this mode does NOT support time on screen, it is directly from the camera.\n\
-h                   help (this help)\n\
-l int               blank last int lines to blank random noise in Logitech quickcam 2500 VGA output.\n\
-m int               motion_detection sensitivity, creates semaphore file ~/jpcat.motion if motion,\n\
                      user must take action if that file exists, and then erase that file,\n\
                      only works if -y flag specified too. Try 10000 as value, more is less sensitive.\n\
-o                   output only on motion.\n\
-p                   grab a single frame and write it to stdout in ppm format, this mode supports time display with -t.\n\
-r                   do not draw the light measure box and on screen info in -a.\n\
-q                   quiet mode.\n\
-s                   spot percentage of screen light measurement, default=20%%\n\
-t                   show time in yuv output\n\
-u                   display in UTC time\n\
-v                   debug, prints functions and arguments\n\
-w int               contrast (white)\n\
-x                   show X windows display\n\
-y                   send mjpeg tools yuv format to stdout\n\
",\
dframes_per_second);
fprintf(stderr, "Examples:\n\
Display in X:\n\
jpcat -x\n\
Display using mplayer:\n\
jpcat -y | mplayer -\n\
Display using xine with time on screen:\n\
jpcat -y -t | xine -D -g stdin:/\n\
Record to H264 using ffmpeg as encoder at 2 frames per second for security with auto exposure on:\n\
nice -n 15 jpcat -r -t -a -u -x -f 2 -y 2>/dev/zero |  nice -n 15  ffmpeg -f yuv4mpegpipe -i - -f avi -vcodec h264 -b 10 -g 300 -bf 2 -y test.avi 2>/dev/zero 1>/dev/zero\n\
Grab a single frame and write it to stdout in ppm format with time added:\n\
jpcat  -t -p > myframe.ppm\n\
This will record an avi movie with only the frames were motion was, with timesteps in the picture, makes it sometimes easier to find events, and keeps the avi files small:\n\
jpcat -t -y -o -m 10000 | ffmpeg -f yuv4mpegpipe -i - -f avi -vcodec h264 -b 10 -g 300 -bf 2 -y test.avi 2>/dev/zero 1>/dev/zero\n\
For the Logitech Quickcam 2500 use manual exposure, and if using kernel 2.6.21 also make sure you have installed the supplied driver, and loaded the driver with modprobe gspca autoexpo=0\n\
jpcat -x -l 12 -b 41 -w 128 -d /dev/video1 -t\n\
");

return 1;
} /* end function print_usage */



int main(int argc, char *argv[])
{
char *video_device = NULL;
int grabmethod = 1;
int format = VIDEO_PALETTE_YUV420P;
char temp[4096];
int a, i;
int err = 0;
int iframe = 0;
unsigned char *pictureData = NULL;
struct frame_t *headerframe;
int yuv_buffer_length;
int y_difference, u_difference, v_difference;
int spot_x_size, spot_y_size, spot_x_start, spot_x_end, spot_y_start, spot_y_end;
int contrast_lock = 0;
int black_level_lock = 0;
unsigned long lstart_usecs, lcurrent_usecs, ldiff_usecs;
//double dfps;
int r, g, b;
double y, u, v;
double cr, cg, cb, cu, cv;
int u_time;
unsigned char *y_ptr, *u_ptr, *v_ptr;
int xa, ya, pos, pos2;
unsigned char *picture = NULL;
int width, height;
static int window_open_flag;
int color_depth;
int dargc = 1;
char *dargv[1] = {"Panteltje's lpcat-0.1.2"};
int have_tried_flag = 0;
unsigned char *psrc, *pdst;
int change_level = 0; // -Wall
int diff;
unsigned char *y_ptr_p, *u_ptr_p, *v_ptr_p;
int motion_detected = 0;
float avgY;
int avgYcount;
int threshold;
FILE *fptr;
int on_screen_info_flag;
int restarts;
int old_restarts;
int displayed_info_frames;
double dmilli_seconds_per_frame;
double dreal_frames_per_second;
char *ptr;
int grab_single_frame_and_output_jpeg_flag;
int grab_single_frame_and_output_ppm_flag;
int output_only_on_motion_flag;
int auto_exposure_flag;
int blank_bottom_lines;

/* get user info */
userinfo = getpwuid(getuid() );

/* get home directory */
home_dir = strsave(userinfo -> pw_dir);
user_name = strsave(userinfo -> pw_name);

/* defaults */
debug_flag = 0;
video_device = strsave("/dev/video0");
if(! video_device)
	{
	fprintf(stderr, "jpcat: main(): could not allocate space for video_device %s, aborting.\n", "/dev/video");

	exit(1);
	}

color_depth = 32;
//width = 352;
//height = 288;

width = 640;
height = 480;

sprintf(temp, "%s/jpcat.motion", home_dir);
semaphore_filename = strsave(temp);
if(! semaphore_filename)
	{
	fprintf(stderr, "jpcat: main(): could not allocate space for semaphore_filename, aborting.\n");
	
	exit(1);
	}
	
auto_contrast_flag = 0;
time_flag = 0;
yuv_output = 0;
spot_percentage = 20;
clock_position = 4;
brightness = 42;	
contrast = 128;
x_display = 0;
dframes_per_second = DEFAULT_FRAME_SPEED;
restarts = 0;
on_screen_info_flag = 1;
old_restarts = -1;
displayed_info_frames = 0;
quiet_flag = 0;
dreal_frames_per_second = 0.0; // -Wall
grab_single_frame_and_output_jpeg_flag = 0;
grab_single_frame_and_output_ppm_flag = 0;
output_only_on_motion_flag = 0;
auto_exposure_flag = 1;
blank_bottom_lines = 0;
/* end defaults */

//fprintf(stderr, " %s video_device=%s\n", version, video_device);

fprintf(stderr, "panteltje jpcat-%s\n", PVERSION);


/* get command line options */
while(1)
	{
	a = getopt(argc, argv, "ab:c:d:e:f:ghl:m:opqrs:tuvw:xy");
	if(a == -1) break;

	switch(a)
		{
		case 'a': // auto contrast
			auto_contrast_flag = 1;
			break;
		case 'b': // brightness
			a = atoi(optarg);
			if( (a < 0) || (a > 255) )
				{
				fprintf(stderr, "jpcat: main(): -b %d brightness: ERROR use value between 0 and 255, aborting.\n", a);

				exit(1);
				}
			brightness = a;
			break;
		case 'd': // video device
			free(video_device);
			video_device = strsave(optarg);
			break;
		case 'e': // auto exposure control
			a = atoi(optarg);
			if(a) auto_exposure_flag = 1;
			else auto_exposure_flag = 0;
			break;
		case 'f': // set frames per second for YUV output (mplayer will display in that speed)
			dframes_per_second = atof(optarg);
			break;
		case 'g': // single frame to stdout in jpeg format */
			grab_single_frame_and_output_jpeg_flag = 1;
			break;
		case 'c': // clock position
			clock_position = atoi(optarg);
			break;
		case 'h': // help
			print_usage();
			exit(1);
			break;
		case 'l': // blank n bottom lines
			blank_bottom_lines = atoi(optarg);
			break;
		case 'm':
			motion_detection_threshold = atoi(optarg);
			if( (motion_detection_threshold < 0) || (motion_detection_threshold > 65535) )
				{
				fprintf(stderr,\
				"jpcat: main(): -m %d motion_detection_threshold: ERROR use value between 1 and 65535,\n\
				aborting.\n", motion_detection_threshold);
				
				exit(1);
				}
			break;
		case 'o':// output only on motion 
			output_only_on_motion_flag = 1;
			break;
		case 'p': // single frabe to stdout in ppm format wit htime and date added.
			grab_single_frame_and_output_ppm_flag = 1;
			break;
		case 'q': // quiet mode
			quiet_flag  = 1;
			break;
		case 'r':
			on_screen_info_flag = 0;
			break;
		case 's': // spot percentage
			spot_percentage = atoi(optarg);
			if( (spot_percentage < 1) || (spot_percentage > 99) )
				{
				fprintf(stderr,\
				"jpcat: main(): -s %d spotsize percent: ERROR use value between 1 and 99, aborting.\n",\
				spot_percentage);

				exit(1);
				}
			break;
		case 't': // show time in display
			time_flag = 1;
			break;
		case 'u': // display in UTC time
			use_utc_flag = 1;
			break;
		case 'v': // verbose / debug
			debug_flag = 1;
			break;
		case 'w': // contrast
			a = atoi(optarg);
			if( (a < 0) || (a > 255) )
				{
				fprintf(stderr,\
				"jpcat: main(): -w %d contrast: ERROR use value between 0 and 255, aborting.\n",\
				a);
				exit(1);
				}
			contrast = a;
			break;
		case 'x': // x windows display
			x_display = 1;
			break;
		case 'y': // enable mjpeg tools yuv output to stdout.
			yuv_output = 1;
			break;
		case -1:
        	break;
		case '?':
			if (isprint(optopt) )
 				{
 				fprintf(stderr, "Unknown option `-%c'.\n", optopt);
 				}
			else
				{
				fprintf(stderr, "Unknown option character `\\x%x'.\n", optopt);
				}
			print_usage();

			exit(1);
			break;			
		default:
			print_usage();

			exit(1);
			break;
		} /* end switch a */
	} /* end while getopt() */


/* sanity checks */
if(grab_single_frame_and_output_ppm_flag && grab_single_frame_and_output_jpeg_flag)
	{
	fprintf(stderr ,"jpcam: syntax error: -g and -p flag both specified, use only one at the time, aborting.\n"); 
	
	exit(1);
	}

if(motion_detection_threshold)
	{
	if(grab_single_frame_and_output_ppm_flag || grab_single_frame_and_output_jpeg_flag)
		{
		fprintf(stderr, "jpcat: syntax error: no motion detection with single picture output, use -y and pipe via ffmpeg, aborting.\n");

		exit(1);		
		}
	} /* end if motion_detection_threshold */
	
if(yuv_output)
	{
	if(grab_single_frame_and_output_ppm_flag)
		{
		fprintf(stderr ,"jpcam: syntax error: -p and -y flag both specified, use only one at the time, aborting.\n"); 
	
		exit(1);
		}

	if(grab_single_frame_and_output_jpeg_flag)
		{
		fprintf(stderr ,"jpcam: syntax error: -g and -y flag both specified, use only one at the time, aborting.\n"); 
	
		exit(1);
		}

} /* end if yuv_output */


/* in case auto contrats start here */
if(auto_contrast_flag)
	{
	brightness = 128;
	contrast = 4;
	}

/* end sanity checks */

dmilli_seconds_per_frame = 1000.0 / (double)dframes_per_second;

signal (SIGALRM, catch_alarm);

while(1) /* restarts */
	{
	restart_flag = 0; /* set by catch_alarm */
	
	memset (&videoIn, 0, sizeof (struct vdIn));
	
	if (init_videoIn(&videoIn, video_device, width, height, format, grabmethod) != 0)     
		{
		fprintf(stderr, "encore rate !!\n");
		}
		

	set_auto_exposure(&videoIn, auto_exposure_flag);


	/* color standard */
	/* Y spec */
	cr = 0.3;
	cg = 0.59;
	cb = 0.11;
	
	/* U spec */
	cu = .5 / (1.0 - cb);
	
	/* V spec */
	cv = .5 / (1.0 - cr);
	
	/* for frames per second */
	if(! start_timeval)
		{
		start_timeval = malloc(sizeof(struct timeval) );
		if(! start_timeval)
			{
			fprintf(stderr, "jpcat: main(): could not allocate space for start_timeval, aborting.\n");
	
			exit(1);
			}
		}
	
	if(! current_timeval)
		{
		current_timeval = malloc(sizeof(struct timeval) );
		if(! current_timeval)
			{
			fprintf(stderr, "jpcat: main(): could not allocate space for current_timeval, aborting.\n");
	
			exit(1);
			}
		}
	
	while(1) /* read frames */
		{
		if(debug_flag)
			{
			fprintf(stderr, "I am the GRABBER !!!!! \n");
			}
			
		SpcaSetBrightness(&videoIn, brightness);
		SpcaSetContrast(&videoIn, contrast);
	
	//fprintf(stderr, "WAS 0 restart_flag=%d restarts=%d\n", restart_flag, restarts);
	
		/* sometimes with error -18 in module no return from v4lGrab(), using alarm to restart the systemn after 1 second */
		err = v4lGrab(&videoIn);
	
	//fprintf(stderr, "WAS past v4lGrab, restart_flag=%d\n", restart_flag);
	
		alarm(10);
	
		if(restart_flag)
			{
			close_v4l(&videoIn);
	
			restarts++;
	
			fprintf(stderr, "jpcat: main(): restart, restarts=%d\n", restarts);	
	
			break;
			}
	
		if(!videoIn.signalquit || (err < 0))
			{
			fprintf(stderr, "jpcat: main(): v4lGrab error=%d, aborting.\n", err);
	
			exit(1);
			}
	
		iframe = (videoIn.frame_cour + (OUTFRMNUMB - 1)) % OUTFRMNUMB; //set the last frame available
	
		videoIn.framelock[iframe]++;
	
		headerframe = (struct frame_t*)videoIn.ptframe[iframe];
	
		pictureData = videoIn.ptframe[iframe] + sizeof(struct frame_t);
	
	//	fprintf(stderr, "WAS*********************iframe=%d\n", iframe);
	
		//size_t fwrite(const void *ptr, size_t size, size_t nmemb, FILE *stream);

//int framesizeIn
//int sizenative; // available size in jpeg		seems always zero.
//		fprintf(stderr, "WAS videoIn.framesizeIn=%d videoIn.sizenative=%d\n", videoIn.framesizeIn, videoIn.sizenative);
	
		if(grab_single_frame_and_output_jpeg_flag)
			{
			fwrite(pictureData, sizeof(char), videoIn.framesizeIn, stdout);

			exit(0);
			}
			

		width = headerframe -> w;
		height = headerframe -> h;
	
		if(! picture)
			{
			picture = (unsigned char *)malloc(width * height * 3);
			if(! picture)
				{
				fprintf(stderr, "jpcat: could not allocate space for picture, aborting\n.");
	
				exit(1);
				}
			}
				
		/* decode the jpeg to rgb24 */
		jpeg_decode(&picture, pictureData, &width, &height);
	
		/* picture is one frame in RGB 24 format */
	
		if(debug_flag)
			{
			fprintf(stderr,\
			"after jpeg_decode, pictureData=%p picture=%p width=%d height=%d\n",\
			pictureData, picture, width, height);
			}
	
		/* if need to blank the last few lines to black out the Logitech Quickcam 2500 random lines bug */
		if(blank_bottom_lines)
			{
			for(ya = 0; ya < height; ya++)
				{
				for(xa = 0; xa < width; xa++)
					{			
					if(ya > (height - blank_bottom_lines) )
						{
						pos = ( (ya * width) * 3) + (xa * 3);
	
						picture[pos] = 0; // b
						picture[pos + 1] = 0; // g
						picture[pos + 2] = 0; // r
						} /* end if in bottom part of screen */						
					} /* end of xa */
				} /* end for ya */
			} /* end if blank_bottom_lines */

		if(x_display)
			{
			/* open a window for the size we now know */
			if(!window_open_flag)
				{
				openwin(dargc, dargv, width, height);
				window_open_flag = 1;
				}
				
			/* get the xbuffer */
			xbuffer = (char *)getbuf();
			if(! xbuffer)
				{
				fprintf(stderr, "jpcat: main(): could not get xbuffer, aborting\n");
				
				exit(1);
				}
	
			if(debug_flag)
				{
				fprintf(stderr, "xbuffer=%p\n", xbuffer);
				}
	
			/* set color_depth (if color_depth_mode == 0) */
			color_depth = wingetdepth();
	
			if(debug_flag)
				{
				fprintf(stderr, "jpcat: main(): colordepth=%d\n", color_depth);
				}
	
			} /* end if x_display */
	
		/* here normal YUV with separated buffers 1/2 */
			
		if(debug_flag)
			{
			fprintf(stderr,\
			"headerframe->w=%d headerframe->h=%d headerframe->size=%d pictureData=%p\n",\
			headerframe -> w, headerframe -> h, headerframe -> size, pictureData);
			}
	
		/* pre-calculate measurement window boundaries, all integer math */
		a = spot_percentage * width;	
		spot_x_size = a / 100; 
		spot_x_start = (width / 2) - (spot_x_size / 2);
		spot_x_end = spot_x_start + spot_x_size;
		
		a = spot_percentage * height;	
		spot_y_size = a / 100;
		spot_y_start = (height / 2) - (spot_y_size / 2);
		spot_y_end = spot_y_start + spot_y_size;
	
		if(debug_flag)
			{
			fprintf(stderr,\
			"spot_percentage=%d\n\
			spot_x_size=%d spot_x_start=%d spot_x_end=%d\n\
			spot_y_size = %d spot_y_start=%d spot_y_end=%d\n",\
			spot_percentage,\
			spot_x_size, spot_x_start, spot_x_end,\
			spot_y_size, spot_y_start, spot_y_end);
			}
	
		if(auto_contrast_flag)
			{
			peak_white = 0;
			bottom_black = 255;
				
			for(ya = 0; ya < height; ya++)
				{
				for(xa = 0; xa < width; xa++)
					{			
					/* test if in measurement box */
					if( (ya > spot_y_start) && (ya < spot_y_end) )
						{
						if( (xa > spot_x_start) && (xa < spot_x_end) )
							{
							pos = ( (ya * width) * 3) + (xa * 3);
	
							b = (int)picture[pos];
							g = (int)picture[pos + 1];
							r = (int)picture[pos + 2];
	
							/* convert to YUV */
			
							/* test yuv coding here */
							y = cr * r + cg * g + cb * b;
	
							if(y > peak_white) peak_white = y;
							if(y < bottom_black) bottom_black = y;
							}
						}						
	
					/* black over whole picture? */
	
					} /* end of xa */
				} /* end for ya */
	
			/* low light warning to rgb24 buffer if could not crank up contrast enough */
			if( (peak_white < 240) && (contrast == 255) )
				{
				/*
				int text_to_rgb24_buffer(\
				unsigned char *text,\
				int x, int y,\
				int fr, int fg, int fb,\
				int br, int bg, int bb,\
				int xsize, int ysize, char *buffer)
				*/                
	
				text_to_rgb24_buffer((unsigned char *)"Low Light", 10, 10,  255, 0, 0,  0, 0, 0, width, height, (char *)picture);
				} /* end if low light */
	
			if(on_screen_info_flag)
				{
				sprintf(temp, "blk=%d wht=%d br=%d ctr=%d fps=%.2f",\
				bottom_black, peak_white, brightness, contrast, dreal_frames_per_second); 	 
				text_to_rgb24_buffer((unsigned char *)temp, 10, 0,  0, 240, 0,  0, 0, 0,  width, height, (char *)picture);
					
				if(motion_detected)
					{
					text_to_rgb24_buffer((unsigned char *)"M", 340, 0,  240, 240, 0,  0, 0, 0,  width, height, (char *)picture);
					}
				}
	
			} /* end if auto contrast flag */
	
		if(time_flag)
			{
			if(! add_clock((char *)picture, width, height) )
				{
				fprintf(stderr, "jpcat: main(): add_clock() failed, aborting.\n");
	
				exit(1);
				}
			}
	
		if(restarts != old_restarts)
			{
			if(! grab_single_frame_and_output_ppm_flag) /* do not want 'restart=0' text in single frame grab */ 
				{
				sprintf(temp, "restart %d", restarts);
				text_to_rgb24_buffer((unsigned char *)temp, 100, 20,  240, 240, 240,  0, 0, 0,  width, height, (char *)picture);
	
				displayed_info_frames++;
				if( displayed_info_frames == 25)
					{
					old_restarts = restarts;
					displayed_info_frames = 0;
					}
				}
			}

		if(! grab_single_frame_and_output_ppm_flag)
			{
			/* wait until time to display picture */
			while(1)
				{
				/* get elapsed time */
				gettimeofday(current_timeval, NULL);
	
				/* calculate time since previous picture */
				lcurrent_usecs =\
				current_timeval -> tv_usec + (1000000 * current_timeval -> tv_sec);
	
				lstart_usecs =\
				start_timeval -> tv_usec + (1000000 * start_timeval -> tv_sec);
	
				ldiff_usecs = lcurrent_usecs - lstart_usecs;
	
				/* test if one frame time elapsed */
				if(ldiff_usecs >= (dmilli_seconds_per_frame * 1000.0) ) break;
	
				usleep(1000);
				} /* end while wait for frame */
	
			/* mark the time the current frame was displayed */
			gettimeofday(start_timeval, NULL);
			
			/* calculate real frames per second */
			dreal_frames_per_second = 1000000.0 / (double)ldiff_usecs;
	
			/* warn if system too slow */
			if(! quiet_flag && !output_only_on_motion_flag)
				{
				if(dreal_frames_per_second < (dframes_per_second * .75) )
					{
					ptr = get_time_as_string(0, 0);
	
					fprintf(stderr,\
					"jpcat: WARN %s current frames per second=%.2f this is < 75%% of the requested %.2f,\n\
	try a lower value with -f flag.\n",\
					ptr, dreal_frames_per_second, dframes_per_second);
	
					free(ptr);
					}
				} /* end while wait for time to display frame */

			} /* end if ! grab_single_frame_and_output_ppm_flag */

		if(yuv_output)
			{
			/* reserve space for uv matrix */
			double up[width][width];
			double vp[width][width];
	
			yuv_buffer_length = \
				(width * height) + ( (width / 2) * (height / 2) ) + ( (width / 2) * (height / 2) );
	
			if(! yuv_buffer)
				{
				yuv_buffer = (unsigned char *)malloc(yuv_buffer_length);		
				if(! yuv_buffer)
					{
					fprintf(stderr, "jpcat: could not allocate space for yuv_buffer, aborting\n.");
	
					exit(1);
					}
				}
	
			/* send MJPEG tools header */
			if(! yuv_header_send_flag)
				{
				/* send the mjpeg tools yuv stream header */
	
				/* stream stream header to get width and height and interlace */
	
				/*
				YUV4MPEG2 W720 H576 F25:1 I? A0:0 XM2AR002
				FRAME
				data
				*/
	
				fprintf(stdout,\
				"YUV4MPEG2 W%d H%d F%d:1 I? A0:0 XM2AR002\n", width, height, (int)dframes_per_second);
	
				yuv_header_send_flag = 1;
				}
	
			y_ptr = yuv_buffer;
			u_ptr = yuv_buffer + (width * height);
			v_ptr = u_ptr + ( (width / 2) * (height / 2) );
	
			/* RGB to YUV */
	
			pos = 0;
			for(ya = 0; ya < height; ya++)
				{
				u_time = 0;
				for(xa = 0; xa < width; xa++)
					{  
					pos = ( (ya * width) * 3) + (xa * 3);
	
					b = (int)picture[pos];
					g = (int)picture[pos + 1];
					r = (int)picture[pos + 2];
	
					/* convert to YUV */
	
					/* test yuv coding here */
					y = cr * r + cg * g + cb * b;
	
					*y_ptr = y;
					y_ptr++;
	
					u = cu * (b - y);
					v = cv * (r - y);
	
					if(ya % 2)
						{
						if(xa % 2)
							{
							*u_ptr = ( (up[ya - 1][xa - 1] + up[ya - 1][xa] + up[ya][xa -1] + u ) / 4.0) + 128; 
							u_ptr++;
							} /* end if odd pixel */
						else
							{
							*v_ptr = ( ( vp[ya - 1][xa - 1] + vp[ya - 1][xa] + vp[ya][xa -1] +  v) / 4.0) + 128;
							v_ptr++;
							}
				
						} /* end if odd pixel */
	
					/* fill uv matrix for lookback */
					up[ya][xa] = u;
					vp[ya][xa] = v;
	
					} /* end for xa */
	
				} /* end for ya */
	
			if(debug_flag)
				{
				fprintf(stderr,\
				"bottom_black=%d peak_white=%d bright %05d=contrast=%05d\n",\
				bottom_black, peak_white, headerframe->bright, headerframe->contrast);
				}
	
			/* motion detection */
			if(motion_detection_threshold)
				{
				motion_detected = 1;
				threshold = motion_detection_threshold;
				
				if(! yuv_buffer_prev)
					{
					yuv_buffer_prev = (unsigned char *)malloc(yuv_buffer_length);
					if(! yuv_buffer_prev)
						{
						fprintf(stderr, "jpcat: could not allocate space for yuv_buffer_prev, aborting\n.");
						
						exit(1);
						}
					}
	
				y_difference = 10;
				u_difference = 10;
				v_difference = 10;
	
				/* compare current yuv buffer to previous one */
				y_ptr = yuv_buffer;
				u_ptr = yuv_buffer + (width * height);
				v_ptr = u_ptr + ( (width / 2) * (height / 2) );
			
				y_ptr_p = yuv_buffer_prev;
				u_ptr_p = yuv_buffer_prev + (width * height);
				v_ptr_p = u_ptr_p + ( (width / 2) * (height / 2) );
	
				/* set for no change */
				change_level = 0;
	
				/* compare new to old y matrices */
				avgY = 0;
				avgYcount = 1;
				for(i = 0; i < width * height; i++)
					{
					avgY = (avgY * (avgYcount - 1) + *y_ptr) / avgYcount;
					avgYcount++;
					diff = *y_ptr - *y_ptr_p;
					y_ptr++;
					y_ptr_p++;
					if( abs(diff) > y_difference)
						{
						change_level++;
						}
					}
	
				if(debug_flag)
					{
					fprintf(stderr,"AverageY=%f ",avgY);
					}
					
				if(avgY <= 90 )
					{
					//increase threshold if it is getting dark - it will take more to register a motion
					threshold += 20000. * (90. -avgY) / 55.; //increase by 0-20000 as it gets darker
					}
	
				if(avgY > 35)
					{ //if it light enough
					/* compare new to old u matrices */
					for(i = 0; i < (width / 2) * ( height / 2); i++)
						{
						diff = *u_ptr - *u_ptr_p;
						if( abs(diff) > u_difference)
							{
							change_level++;
							}
						}
	
					/* compare new to old v matrices */
					for(i = 0; i < (width / 2) * ( height / 2); i++)
						{		
						diff = *v_ptr - *v_ptr_p;
						if( abs(diff) > v_difference)
							{
							change_level++;
							}
						}
					
					if(debug_flag)
						{
						fprintf(stderr, "process_jpeg_buffer(): motion_detection: change_level=%d\n", change_level); 
						}
	
					/* set flag if treshhold exceeded */
					if(change_level < threshold) motion_detected = 0;
					}
				else
					{
					//if it is dark, don't detect motion
					motion_detected = 0;
					}
	
	//			if( motion_detected )fprintf(stderr,"Motion detected, change_level=%d\n",change_level);
	//			else fprintf(stderr,"  change_level=%d\n",change_level);
	
				if(debug_flag)
					{
					if(motion_detected) fprintf(stderr, "process_jpeg_buffer(): \n\nmotion detected!\n");
					else fprintf(stderr, "process_jpeg_buffer(): no motion detected.\n");
					}	
	
				/* save values so we can compare to those next time */ 
				memcpy(yuv_buffer_prev, yuv_buffer, yuv_buffer_length);
	
				if(motion_detected)
					{
					/* create semaphore file if motion detected and filename specified */
					if(semaphore_filename)
						{		
						fptr = fopen(semaphore_filename, "w");
						if(! fptr)
							{
							fprintf(stderr,\
							"jpcat: ERROR could not create semaphore file %s  aborting.\n", semaphore_filename);
	
							exit(1);
							}
	
						fclose(fptr);
						} /* end if semaphore_filename */
					} /* end if motion_detected */ 
	
				/* save values so we can compare to those next time */ 
				memcpy(yuv_buffer_prev, yuv_buffer, yuv_buffer_length);
	
				} /* end if motion_detection_threshold */

			if(output_only_on_motion_flag && motion_detected)
				{
				if(! quiet_flag)
					{
					fprintf(stderr, "jpcat: motion detected, change_level=%d\n", change_level);
					}
				}
				
			if( (! output_only_on_motion_flag) || (output_only_on_motion_flag && motion_detected) ) /* only output mjpegtools yuv frame if motio nwas detected and motion detection was requested */
				{ 
				/* output an mjpeg tools yuv frame header */
				/*
				FRAME-HEADER consists of
				 string "FRAME "  (note the space after the 'E')
				 unlimited number of ' ' separated TAGGED-FIELDs
				 '\n' line terminator
				*/
	
				fprintf(stdout, "FRAME\n");
	
				/* output an mjpeg tools yuv frame */
				/* write Y */
				/* write Cb */
				/* write Cv */
	
				if(debug_flag)
					{
					fprintf(stderr, "yuv frame length=%d\n", yuv_buffer_length);
					}
	
				a = fwrite(yuv_buffer, sizeof(char), yuv_buffer_length, stdout); 
				if(a != yuv_buffer_length)
					{
					fprintf(stderr, "jpcat: output yuv: could only write %d of %d bytes, aborting.\n", a, yuv_buffer_length);
	
					exit(1);
					}
	
				} /* end if yuv_output */
	
			} /* end ! output_only_on_motion_flag or output_only_on_motion_flag && motion_detected */	


		if(auto_contrast_flag)
			{	
			if(on_screen_info_flag)
				{
				/* spot box only to local display, not to YUV out and recording */
				/* only draw the box while not locked */	
				if( (! contrast_lock) || (! black_level_lock) )
					{		
					/* draw measurement box in RGB screen (could do there only if after yuv processing) */
					for(ya = spot_y_start; ya < spot_y_end; ya++) 
						{
						pos  = (ya * width * 3) + (spot_x_start * 3);
						pos2 = (ya * width * 3) + (spot_x_end   * 3);
						if(ya % 2)
							{
							picture[pos    ] = 245; 
							picture[pos + 1] = 245;
							picture[pos + 2] = 245;
	
							picture[pos2    ] = 245; 
							picture[pos2 + 1] = 245;
							picture[pos2 + 2] = 245;
							}						
						else
							{
							picture[pos    ] = 15;
							picture[pos + 1] = 15;
							picture[pos + 2] = 15;
	
							picture[pos2    ] = 15;
							picture[pos2 + 1] = 15;
							picture[pos2 + 2] = 15;
							}
						}
					for(xa = spot_x_start; xa < spot_x_end; xa++) 
						{
						pos  = (spot_y_start * width * 3) + (xa * 3);
						pos2 = (spot_y_end   * width * 3) + (xa  * 3);
						if(xa % 2)
							{
							picture[pos    ] = 245; 
							picture[pos + 1] = 245;
							picture[pos + 2] = 245;
	
							picture[pos2    ] = 245; 
							picture[pos2 + 1] = 245;
							picture[pos2 + 2] = 245;
							}						
						else
							{
							picture[pos    ] = 15;
							picture[pos + 1] = 15;
							picture[pos + 2] = 15;
	
							picture[pos2    ] = 15;
							picture[pos2 + 1] = 15;
							picture[pos2 + 2] = 15;
							}
						}		
					} /* end if draw on_screen_info_flag */
				} /* end if not contrast locked */
	
			if(peak_white < 240)
				{
				if(contrast < 255) contrast++;
				contrast_lock = 0;
				}
			else if(peak_white > 250)
				{
				if(contrast > 0) contrast--;	
				contrast_lock = 0;
				}
			else
				{
				/* between 240 and 250 good range */
				// do nothing
				contrast_lock = 1;
				}
	
			if(bottom_black > 20)
				{
				if(brightness > 0) brightness--;
				black_level_lock = 0;
				}
			else if(bottom_black < 10)
				{
				if(brightness < 255) brightness++;
				black_level_lock = 0;
				}
			else
				{
				/* between 1- and 20 do nothing */
				// do nothing		
				black_level_lock = 1;
				}
	
			} /* end if auto_contrast_flag */
			
		if(x_display)
			{
			/* rgb24 to xbuffer rgb32 */
			psrc = picture;
			pdst = (unsigned char *)xbuffer;
			for(ya = 0; ya < height; ya++)
				{
				for(xa = 0; xa < width; xa++)
					{
					b = *psrc++;
					g = *psrc++;
					r = *psrc++;
	
					if( (color_depth == 24) || (color_depth == 32) )
						{
						*pdst++ = b;
						*pdst++ = g;
						*pdst++ = r;
	
						if(color_depth == 32)
							{
							/* set unsused byte to zero */
							*pdst++ = 0;					
							}
								
						} /* end if color_depth 24 or 32 */
					} /* end if xa */
				} /* end for ya */
	
			/* display the image */
			putimage(width, height);
	
			/* on the very first image a white screen is the result, this retries */
			if(! have_tried_flag)
				{
				putimage(width, height);
	
				have_tried_flag = 1;
				}
			} /* end if x_display */
	
		if(grab_single_frame_and_output_ppm_flag)
			{
			/* write the ppm header */
			fprintf(stdout, "P6\n%i %i\n255\n", width, height);

			/* need vertical flip, but not horizontal flip */
			psrc = picture;
			for(ya = 0; ya < height; ya++)
				{
				for(xa = 0; xa < width; xa++)
					{
					b = *psrc++;
					g = *psrc++;
					r = *psrc++;

					fprintf(stdout, "%c%c%c", r, g, b);
					} /* end for all xa */
				} /* end for ya (all lines) */

			
			exit(0);
			} /* end if grab_single_frame_and_output_ppm_flag */

		videoIn.framelock[iframe]--;

    } /* end while read pics */
} /* end while restarts */	

exit(1);
} /* end function main */

